(function() {
  var Parser, Result, ex, specification, type;
  var __hasProp = Object.prototype.hasOwnProperty;
  ex = require('./exceptions');
  type = require('./types');
  specification = require('./specification');
  Result = (function() {
    function Result(parser) {
      this.parser = parser;
      this.argv = [];
      this.given = {};
    }
    Result.prototype._arg = function(blob) {
      return this.argv.push(blob);
    };
    Result.prototype._flag = function(name) {
      var target;
      if (!this.parser.isFlag(name)) {
        throw new ex.NoValue(name);
      }
      target = this.parser.index[name];
      return this.given[target] = true;
    };
    Result.prototype._param = function(name, value) {
      var match, o, p, spec, target, _base, _base2;
      if (!value) {
        throw new ex.NoValue(name);
      }
      p = this.parser;
      target = p.index[name];
      if (!target) {
        throw new ex.UnknownOption(name);
      }
      spec = p.spec[target];
      switch (spec.type) {
        case type.flag:
          throw new ex.FlagWithValue({
            name: name,
            value: value
          });
          break;
        case type.scalar:
          if (target in this.given) {
            throw new ex.ScalarWithValues(name);
          }
          return this.given[target] = value;
        case type.array:
          o = (_base = this.given)[target] || (_base[target] = []);
          return o.push(value);
        case type.object:
          match = /^([^=]*)(?:=(.+))?$/.exec(value);
          o = (_base2 = this.given)[target] || (_base2[target] = {});
          return o[match[1]] = match[2];
        default:
          throw new ex.UnvalidatedType(name);
      }
    };
    Result.prototype._finalize = function() {
      var p, spec, target, v, _ref;
      p = this.parser;
      this.values = {};
      _ref = p.spec;
      for (target in _ref) {
        if (!__hasProp.call(_ref, target)) continue;
        spec = _ref[target];
        if (spec.type === type.flag) {
          v = !!this.given[target];
        } else if (target in this.given) {
          v = this.given[target];
        } else {
          v = spec["default"];
        }
        if (spec.required && ((spec.type === type.scalar && !v) || (spec.type === type.array && v.length < 1))) {
          throw new ex.RequiredMissing(target);
        }
        this.values[target] = v;
        if (!(target in this && target.charAt(0) !== '_')) {
          this[target] = v;
        }
      }
      return this;
    };
    return Result;
  })();
  exports.Parser = Parser = (function() {
    function Parser(spec) {
      this.spec = spec;
      this.index = specification.index(this.spec);
    }
    Parser.prototype.isFlag = function(name) {
      var target;
      target = this.index[name];
      if (!target) {
        throw new ex.UnknownOption(name);
      }
      return this.spec[target].type === type.flag;
    };
    Parser.prototype.parse = function(argv) {
      var a, blob, f, first, i, match, name, result, _i, _j, _len, _len2, _ref, _ref2;
      result = new Result(this);
      i = 0;
      while (i < argv.length) {
        blob = argv[i];
        if (blob === '-') {
          result._arg(blob);
        } else if (blob === '--') {
          _ref = argv.slice(i + 1);
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            a = _ref[_i];
            result._arg(a);
          }
          break;
        } else if (match = /^-(\w.+)$/.exec(blob)) {
          name = match[1];
          first = '-' + name.charAt(0);
          if (this.isFlag(first)) {
            _ref2 = name.split('');
            for (_j = 0, _len2 = _ref2.length; _j < _len2; _j++) {
              f = _ref2[_j];
              result._flag('-' + f);
            }
          } else {
            result._param(first, name.slice(1));
          }
        } else if (match = /^(--\w+)=(.*)$/.exec(blob)) {
          result._param(match[1], match[2]);
        } else if (match = /^(-\w)$/.exec(blob) || /^(--\w+)$/.exec(blob)) {
          name = match[1];
          if (this.isFlag(name)) {
            result._flag(name);
          } else {
            result._param(name, argv[++i]);
          }
        } else {
          result._arg(blob);
        }
        i += 1;
      }
      return result._finalize();
    };
    return Parser;
  })();
}).call(this);
